import { defineStore } from "pinia";
import { getDictByCode, getUserFormRole, getRegion } from "@/api/dict"
import { strFirstUpper, treeChildrenEmpty, treeLabelValue } from "@/utils/index"

export const useDictStore = defineStore({
  id: 'DictStroe',
  state: () => {
    return {
      noConstant: ['regionList'],
      tree: [],
      noCache: [],
      params: ['roleUserList'],
      constant: {
        // ...constantData, // 前端维护的常量数据
        regionList: [],
        roleUserList: [], // 角色下用户
        projectInitiationStatus: [], // 融资管理 - 业务状态
        flowableTime: [],
        code: [],
        c_supplier_status: [],
        repaymentMethod: [], // 回款方式
        transferNoticeMethod: [], // 转让通知方式
        serviceScope: [], // 服务范围
        repaymentAdjustment: [], // 还款期限调整方式
        paymentMethodForInterestManagementFee: [], // 还款方式
        currencyType: [], // 币种
        incormeExpenditureType: [], // 收支类型
        idType: [], // 身份类别
        certType: [], // 责任人身份标识类型、配偶证件类型
        quotaDisburseArlpType: [], // 还款责任人类型
        collateralType: [], // 抵押物种类
        collateralIdType: [], // 抵押物识别号类型
        pledgeType: [], // 质押物种类
        invoice_status: [], // 发票管理列表状态
        invoiceType: [], // 发票类型
        amountKind: [], // 金额种类
        salesContractSettleCycleMethod: [], // 结算周期方式
        oaBillPaymentType: [], // 退款管理-付款类型
        principalPaymentMethod: [], // 授信决议 - 本金支付方式
      }
    }
  },
  actions: {
    getConstant(name) {
      const events = []
      const isConstant = name => {
        if (this.noConstant.includes(name)) {
          events.push(this[`get${strFirstUpper(name)}`](name))
        } else if (this.tree.includes(name)) {
          // events.push(dispatch('getTreeDict', name))
        } else if (typeof (name) === 'object' && this.params.includes(name.type)) {
          events.push(this[`get${strFirstUpper(name.type)}`](name))
        } else {
          events.push(this['getDict'](name))
        }
      }
      if (Array.isArray(name)) {
        for (const item of name) {
          var type = typeof (item) === 'object' ? item.type : item
          if (!this.noCache.includes(type)) {
            const constantType = Object.prototype.toString.call(this.constant[type])
            if (constantType === '[object Array]') {
              if (this.constant[type]?.length) continue
            } else if (constantType === '[object Object]') {
              if (Object.keys(this.constant[type]).length) continue
            } else if (constantType === '[object String]') {
              if (this.constant[type]) continue
            }
          }
          isConstant(item)
        }
      } else {
        var typeStrTemp = typeof (name) === 'object' ? name.type : name
        if (!this.noCache.includes(typeStrTemp)) {
          if (!this.noCache.includes(typeStrTemp)) {
            const typeStr = Object.prototype.toString.call(this.constant[typeStrTemp])
            if (typeStr === '[object Array]') {
              if (this.constant[typeStr].length) return
            } else if (typeStr === '[object Object]') {
              if (Object.keys(this.constant[typeStr]).length) return
            } else if (typeStr === '[object String]') {
              if (this.constant[typeStr]) return
            }
          }
        }
        isConstant(name)
      }
      return Promise.all(events)
    },
    getDict(code) {
      return new Promise((resolve, reject) => {
        getDictByCode({ code }).then((res: AxiosResponseRes) => {
          if (res.data) {
            if (code in this.constant) {
              this.constant[code] = res.data
            }
            resolve(res.data)
          }
        }).catch(error => {
          reject(error)
        })
      })

    },
    getRoleUserList(params) {
      return new Promise((resolve, reject) => {
        getUserFormRole(params.query).then((res: AxiosResponseRes) => {
          this.constant[params.type] = res.data
          resolve(res.data)
        }).catch(error => {
          reject(error)
        })
      })
    },
    getRegionList(type) {
      return new Promise((resolve, reject) => {
        getRegion().then((res: AxiosResponseRes) => {
          const list = treeChildrenEmpty(res.data)
          treeLabelValue(list, { value: 'deptCode', label: 'name' })
          this.constant[type] = res.data
          resolve(res.data)
        }).catch(error => {
          reject(error)
        })
      })
    }
  },
})
